## Copyright (C) 2003 Csaba Szepesvari
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

## -*- texinfo -*-
## @deftypefn {Function File} {[@var{r1},@var{r2}]=} FUNNAME (@var{ARG1}, @var{ARG2}, ..)
## INCLUDE HERE A SHORT DESCRIPTION
##
## Default values are @var{a} = 0, @var{b} = 1.
##
## Example usage:
##
## @var{show}=1;
## [@var{r},@var{pr}] = rejection_rnd("normal01_pdf","normal2_pdf","normal2_rnd",1.45,N,@var{show});
## @end deftypefn

## Author: Csaba Szepesvari (www.sztaki.hu/~szcsaba)
## Description: PDF of the uniform distribution
## Keywords: 

function rejection_rnd_test(N,show)

if (nargin==0)
  N = 1000;
  show = 0;
elseif (nargin==1)
  show = 0;
elseif (nargin>2)
  printf("nargin is %i",nargin);
  error("rejection_rnd_test: expecting max 2 arguments!");
endif

[r,pr] = rejection_rnd("normal01_pdf","normal2_pdf","normal2_rnd",1.45,N,show);
rrt_plot(r,pr,"normal01_pdf");
printf("Press a key when you are ready to proceed to the next example\n");
pause;

target_pdf   = funspec("x","y","y = abs(x) .* normal_pdf(x,0,1) ./ 0.786;");
proposal_pdf = funspec("x","y","y = 0.5 * normal_pdf(x,-1,1) + 0.5 * normal_pdf(x,1,1);");
proposal_rnd = funspec("N","y","  r = rand(N,1); l=ones(N,1)*0.5; t = r>l;\n  n1 = sum(t); n2 = N-n1;\n  y=zeros(N,1); if (n1>0) y1 = normal_rnd(-1,1,n1,1); y(find(r>l))=y1;endif;\n  if (n2>0) y2 = normal_rnd(1,1,n2,1); y(find(r<=l))=y2; endif;  ");
[r,pr] = rejection_rnd(target_pdf,proposal_pdf,proposal_rnd,1.2/0.786,N,show);
rrt_plot(r,pr,target_pdf);

endfunction

function rrt_plot(r,pr,target_pdf)
printf("Created a histogram for inspection..\n");
printf("Sampling efficiency ratio (1.0 is perfect): %1.3f\n",pr);
N = length(r);
clg;
hold on;
binnum = max(10,round(N/100));
[nn,xx] = hist(r,binnum,1); 
nn = nn ./ (xx(2)-xx(1));   % normalizing the histogram with the size of the bins
bar(xx,nn);
x = [-5:0.1:5];
y = feval(target_pdf,x);
plot(x,y);
hold off;
endfunction
